
% Filename: ColorDen.m
%
% Operation: denoisinig color images using using the multiband "ProbShrink" method [1],
% which is an extension of the greyscale "GenLik" algorithm of [2].
%
%[1] A. Pizurica and W. Philips, "Estimating probability of presence of a signal of interest
%    in multiresolution single- and multiband image denoising," Technical report, May 2004.%
%
%[2] A. Pizurica, Image denoising using wavelets and spatial context modeling, Ph.D. thesis, 
%    Ghent University, June 2002.%
%
%
% This denoising program uses wavelet decomposition and reconstruction functions
% developed by Rice University, Houston, Texas and its contributors.
%
% This software is distributed and licensed to you on a non-exclusive 
% basis, free-of-charge. Redistribution and use in source and binary forms, 
% with or without modification, are permitted provided that the above 
% copyright notice is clearly maintained.


clear all;
close all;
 
Ref=input('Reference noise-free image available? (y/n) ','s');

% Read reference, noise-free image
if (Ref=='y')
    InputType=0;
    while (InputType~=1 & InputType~=2 & InputType~=3 & InputType~=4)  
     InputType=input('\nReference image format:\n1. "*.tif"\n2. "*.sg2"\n3. "*.mat"\n4. "*.bin"\n>>  ');
    end
    imload;
    Xclean=SCAN;
    clear SCAN;
    [M,N]=size(Xclean(:,:,1));
  
    Choice=input('Choose 1 or 2:\n1. An arbitrary noise variance\n2. A stored noisy image\n ');

    if(Choice==1)
       NoiseStd=input('Standard Deviation of added noise: ');
       Xnoisy=Xclean+randn(M,N,3)*NoiseStd;
       clean_image=input('Reference image name: ','s');
       image_name=strcat(clean_image,'_std',num2str(NoiseStd));
    else
       InputType=0;
       while (InputType~=1 & InputType~=2 & InputType~=3 & InputType~=4)  
         InputType=input('\nNoisy image format:\n1. "*.tif"\n2. "*.sg2"\n3. "*.mat"\n4. "*.bin"\n>>  ');
       end
       imload;
       Xnoisy=SCAN;
       clear SCAN;
  
       %Extract the image name from the filename
       image_name=filename(1:(length(filename)-4))
       SigmaInfo=input('Noise standard deviation known? (y/n) ','s');
       if(SigmaInfo=='y')
           NoiseStd=input('Standard Deviation of added noise: ');
       else
          [clp,chor,cver,cdiag]=dwt2(Xnoisy(:,:,1),'db1');
          NoiseStd=thr_univ(cdiag);
      end
    end
  
else % No reference noise-free image image available

    % Read noisy image
    InputType=0;
    while (InputType~=1 & InputType~=2 & InputType~=3 & InputType~=4)  
        InputType=input('\nNoisy image format:\n1. "*.tif"\n2. "*.sg2"\n3. "*.mat"\n4. "*.bin"\n>>  ');
    end
    imload;
    Xnoisy=SCAN;
    clear SCAN;
    imagesc(uint8(Xnoisy));
    colormap(gray);

    %Extract the image name from the filename
    image_name=filename(1:(length(filename)-4))
    [clp,chor,cver,cdiag]=dwt2(Xnoisy(:,:,1),'db1');
    NoiseStd=thr_univ(cdiag);
    EstimatedNoiseStd=NoiseStd
end
 
[M,N]=size(Xnoisy(:,:,1)); 

figure('Units','normalized','Position',[0.05 0.05 0.38 0.9]);
subplot(2,1,1),imagesc(uint8(Xnoisy));
title('Input image');


%======================================================
% Parameters

% Choose a wavelet
 wtypeN=0;
  while((wtypeN~=1) & (wtypeN~=2) & (wtypeN~=3) & (wtypeN~=4) & (wtypeN~=5))
    wtypeN=input('select a wavelet: 1. (db2), 2. (db4),  3. (sym2), 4. (sym4) or 5. (sym8) ');
 end
 
 if(wtypeN==1)
    wtype='db2'
    [LO_D,HI_D,LO_R,HI_R] = wfilters('db2');
    h=LO_R;
 elseif(wtypeN==2)
    wtype='db4'
    [LO_D,HI_D,LO_R,HI_R] = wfilters('db4');
    h=LO_R;
 elseif(wtypeN==3)
    wtype='sym2'
    [LO_D,HI_D,LO_R,HI_R] = wfilters('sym2');
    h=LO_R;
 elseif(wtypeN==4)
    wtype='sym4'
    [LO_D,HI_D,LO_R,HI_R] = wfilters('sym4');
    h=LO_R;
 elseif(wtypeN==5)
    wtype='sym8'
    [LO_D,HI_D,LO_R,HI_R] = wfilters('sym8');
    h=LO_R;
 end

W=0;

SigIntDEF=1;

SigIntOPT=input('DEFAULT/other signal of interest (any key/other) ','s');

if(SigIntOPT=='other')
   SigIntDEF=input('Multiplication factor for the threshold that define the signal of interest (0.5-1): ');
end

%=====================================================  

Xden=genlikSI_ColorFun(Xnoisy,NoiseStd,W,h,SigIntDEF);
subplot(2,1,2),imagesc(uint8(Xden));
   
   
if(Ref=='y')
  
  MSE=sum(sum(sum((Xclean-Xden).^2)))/(M*N*3);
  MSEin=sum(sum(sum((Xclean-Xnoisy).^2)))/(M*N*3);


PSNR=10*log10(255^2/MSE)
PSNRin=10*log10(255^2/MSEin)
end

SaveResult=input('Save the result in tif format? (y/n)? ','s');
if(SaveResult=='y')
    Filename=strcat(image_name,'_cden.tif');
    imwrite(uint8(Xden),Filename,'tif');
end

   




